//sets speed in the playSound function
let playSpeed = 1; 
 // resamples the audio by multiplying the sample rate in the constructor,
 // this variable can be re-set in playSound funtion 
let resample = 1;


let filterNameType = "lowshelf";
let filterFrequency = 1000;
let filterGain = 25;

let waveShapeCurve = null;
let waveShaperSampling = 'none';

let convolverBypass = "false";

// This class helps manage audio playback code. You may find it useful
// to re-use it in your own projects as well.
export default class AudioManager
{
	constructor(runtime, contextOpts)
	{
		this.runtime = runtime;
		this.audioContext = null;
		
		// Create the AudioContext. Most browsers use AudioContext,
		// but Safari still uses webkitAudioContext. One of the two
		// should always be supported.
		if (typeof AudioContext !== "undefined")
			this.audioContext = new AudioContext({sampleRate: 48000 * resample}, contextOpts);
		else if (typeof webkitAudioContext !== "undefined")
			this.audioContext = new webkitAudioContext({sampleRate: 48000 * resample }, contextOpts);
		else
			throw new Error("AudioContext not supported");
	}
	
	// This wraps the audioContext's decodeAudioData in a promise, since
	// some browsers only support the callback version. Also, unfortunately
	// Safari does not yet have built-in support for decoding WebM Opus files.
	// To work around this Construct provides its own WebM Opus decoder
	// which can be used as a fallback.
	decodeAudioData(arrayBuffer, isWebM)
	{
		// Note if a non-WebM file is passed, always use the built-in decoder.
		if (this.runtime.assets.isWebMOpusSupported || !isWebM)
		{
			// Built-in support for WebM Opus: use built-in decoder,
			// and wrap in a promise so it can be used with 'await'.
			return new Promise((resolve, reject) =>
			{
				this.audioContext.decodeAudioData(arrayBuffer, resolve, reject);
			});
		}
		else
		{
			// No built-in support for WebM Opus: use Construct's decoder
			return this.runtime.assets.decodeWebMOpus(this.audioContext, arrayBuffer);
		}
	}
	
	// Load an AudioBuffer from a project file name e.g. "sfx5.webm".
	async loadSound(url)
	{
		// Check if the given file uses the .webm extension.
		const isWebM = url.toLowerCase().endsWith(".webm");
		
		// Note that media files, including sound and music, are
		// exported under a media folder, so add that to the URL.
		const audioUrl = this.runtime.assets.mediaFolder + url;

		// Ask the runtime to fetch the URL as an ArrayBuffer
		// for decoding. Don't use a normal 'fetch' because it
		// doesn't work in preview mode or on some platforms
		// like Cordova which use local files.
		const arrayBuffer = await this.runtime.assets.fetchArrayBuffer(audioUrl);

		// Once the compressed audio data has been loaded as an
		// ArrayBuffer, decode it to an AudioBuffer ready for playback.
		// Note indicate if the file is WebM since it may need to
		// switch to a fallback if the browser doesn't support WebM.
		const audioBuffer = await this.decodeAudioData(arrayBuffer, isWebM);
		

		return audioBuffer;
	};
	
	updateBiquadFilter(fName, fFreq, fGain)
	{
		filterNameType = fName;
		filterFrequency = fFreq;
		filterGain = fGain;
	}
	
	makeDistortionCurve(amount) 
	{
  		const k = typeof amount === "number" ? amount : 50;
  		const n_samples = 44100;
  		waveShapeCurve = new Float32Array(n_samples);
  		const deg = Math.PI / 180;

  		for (let i = 0; i < n_samples; i++) 
  		{
    		const x = (i * 2) / n_samples - 1;
    		waveShapeCurve[i] = ((3 + k) * x * 20 * deg) / (Math.PI + k * Math.abs(x));
  		}
  		return waveShapeCurve;
	}
	

distortionSampling(samplingString)
{
	waveShaperSampling = samplingString;
}

convolverSettings(bypassState)
{
	convolverBypass = bypassState;

}
  
		
	// Play an AudioBuffer.
	async playSound(audioBuffer, delayStartTime, attack, decay, playSpeed, resample)
	{
		//set up the different effect audio nodes we will use for the app
		const biquadFilter = this.audioContext.createBiquadFilter();
		const waveShaper = this.audioContext.createWaveShaper();
		const convolver = this.audioContext.createConvolver();
		const compressor = this.audioContext.createDynamicsCompressor();
		const source = this.audioContext.createBufferSource();
		
		
		// Grab audio track via fetch() for convolver node
		try 
		{
  			const response = await fetch(
    		"https://mdn.github.io/voice-change-o-matic/audio/concert-crowd.ogg",
  		);
  		const arrayBuffer = await response.arrayBuffer();
  		const decodedAudio = await this.audioContext.decodeAudioData(arrayBuffer);
  		convolver.buffer = decodedAudio;
		} 		
		catch (error) 
		{
			console.error(
    		`Unable to fetch the audio file: ${name} Error: ${err.message}`,
  			);
		}
		
		
		source.buffer = audioBuffer;
		
		// connect nodes to output destination
		source.connect(biquadFilter);
		biquadFilter.connect(waveShaper);
		waveShaper.connect(compressor);
		compressor.connect(this.audioContext.destination);
		source.connect(convolver);
		
		
		if (convolverBypass === "true")
		{
		source.disconnect(convolver);
		}
		else
		{
		// here convolver is set to run in parallel signal 
		convolver.connect(this.audioContext.destination);
		}
		
		source.playbackRate.value = playSpeed;
		source.start(delayStartTime, attack, decay, playSpeed, resample);
		
		// manipuate effects
		biquadFilter.type = filterNameType;
		biquadFilter.frequency.setValueAtTime(filterFrequency, this.audioContext.currentTime);
		biquadFilter.gain.setValueAtTime(filterGain, this.audioContext.currentTime);
		waveShaper.curve = waveShapeCurve;
		waveShaper.oversample = waveShaperSampling;
		
		compressor.threshold.setValueAtTime(-50, this.audioContext.currentTime);
		compressor.knee.setValueAtTime(40, this.audioContext.currentTime);
		compressor.ratio.setValueAtTime(12, this.audioContext.currentTime);
		compressor.attack.setValueAtTime(0, this.audioContext.currentTime);
		compressor.release.setValueAtTime(0.25, this.audioContext.currentTime);
	}


	
	// Load a music track. This uses the <audio> element instead of
	// Web Audio, because music tracks tend to be a lot longer,
	// and Web Audio can only play fully decompressed tracks, whereas
	// the <audio> element can stream tracks. However if the browser
	// does not have built-in support for decoding WebM Opus, instead
	// switch to using loadSound() which can use Construct's decoder.
	async loadMusic(url)
	{
		// Check if the given file uses the .webm extension.
		const isWebM = url.toLowerCase().endsWith(".webm");
		
		// If there is no built-in support for WebM Opus, swap to using
		// loadSound() since it can use Construct's decoder. This is
		// less efficient, but there are no other options for playing
		// WebM Opus audio. Construct's Audio plugin works the same way.
		if (isWebM && !this.runtime.assets.isWebMOpusSupported)
			return this.loadSound(url);
		
		// Otherwise if there is built-in support for WebM Opus continue
		// with creating an <audio> element.
		const audioElem = new Audio();
		
		// Use getMediaFileUrl to get a URL that can be used in the
		// src attribute. Don't set the src directly because it doesn't
		// work in preview mode or on some platforms like Cordova that
		// use local files.
		const musicUrl = await this.runtime.assets.getMediaFileUrl(url);
		audioElem.src = musicUrl;
		
		// Load the audio element, and wait for the canplaythrough event
		// to fire to indicate it's loaded enough to play to the end.
		audioElem.load();
		await this.waitForCanPlayThrough(audioElem);
		
		return audioElem;
	}
	
	// Helper method to return a promise that resolves when the
	// "canplaythrough" event fires, indicating it's ready for playback.
	// It also rejects if the error event fires.
	 waitForCanPlayThrough(audioElem)
	{
		return new Promise((resolve, reject) =>
		{
			audioElem.addEventListener("canplaythrough", resolve);
			audioElem.addEventListener("error", reject);
		});
	}
	
	
	playMusic(audio)
	{
		// loadMusic switches to using Web Audio if the browser does not
		// support WebM Opus. In this case an AudioBuffer will end up
		// passed to this method instead of an <audio> element. If this
		// happens switch to calling playSound() instead.
		if (audio instanceof AudioBuffer)
		{
			this.playSound(audio);
		}
		else
		{
			// Just use the play() method of HTMLMediaElement. It will
			// start streaming playback without fully decompressing the file.
			audio.play();
		}
	}
}