import AudioManager from "./audioManager.js";



// Audio manager class for handling audio playback.
let audioManager = null;
// to store buffer
let whooshSample = null;
// to store sliders
let filterUpdate = null;
let distortionUpdate = null;

runOnStartup(async runtime =>
{
	// add event listener
	runtime.addEventListener("beforeprojectstart", () => OnBeforeProjectStart(runtime));
	
	
	// Initialise the audio manager. See AudioManager.js for details.
	audioManager = new AudioManager(runtime);
	
	// During the loading screen, load both sound files as
	// AudioBuffers and the music track all in parallel, so
	// they are ready for immediate playback on startup.
	//[footstepsAudio1, footstepsAudio2, footstepsAudio3] = await Promise.all([
	[whooshSample] = await Promise.all([
		audioManager.loadSound("Whoosh_SpaceShip_PassBy_Stereo-001.webm")
		
	]);
});

async function OnBeforeProjectStart(runtime)
{
	// get reference to object (slider) in the scene layout
	filterUpdate = runtime.objects.BiquadfilterS.getFirstInstance();
	
	distortionUpdate = runtime.objects.WaveShaperS.getFirstPickedInstance();

	
	// Add an "input" event to each of the sliders to update the effect parameters.
	// (Note "input" fires while dragging the slider, but "change" only fires when
	// the slider stops being dragged.)
	filterUpdate.addEventListener("input", UpdateParameters);
	distortionUpdate.addEventListener("input", UpdateParameters);

}

function UpdateParameters()
{
	// filter options lowpass, highpass, bandpass, lowshelf, highshelf, peaking, notch, allpass
	// definitions at (https://developer.mozilla.org/en-US/docs/Web/API/BiquadFilterNode)
	//updateBiquadFilter(fName, fFreq, fGain)
	audioManager.updateBiquadFilter("lowshelf" ,filterUpdate.value, 25);
	//console.log(filterUpdate.value)
	
	audioManager.makeDistortionCurve(distortionUpdate.value);
	//console.log(distortionUpdate.value);
	
}
export function setDistortionSampling( DISTSampling = '')
{
	audioManager.distortionSampling(DISTSampling);
	//console.log(DISTSampling);
}

export function playSample()
{
//playSound(audioBuffer, delayStartTime, attack, decay, playSpeed, resample_by)
	audioManager.playSound(whooshSample, 0, 0, 2, 1, 1);
}

// bypass convolution with button toggle funtionality
let on =false;
export function BypassConvolver()  
{

	if (on)
	{
		audioManager.convolverSettings("false")
		console.log("convolver on");
		on = false; // toggle back
	}
	else
	{
		audioManager.convolverSettings("true");
		console.log("convolver bypassed");
		on = true; // toggle back
	}
	

	
}










